@@ -8,7 +8,12 @@ load_dotenv(find_dotenv('config.env'))
 
 
 def get_env_int(var_name, default=None):
-    return int(os.environ[var_name]) if os.environ.get(var_name) and os.environ[var_name].isnumeric() else default
+    return (
+        int(os.environ[var_name])
+        if os.environ.get(var_name)
+        and os.environ[var_name].isnumeric()
+        else default
+    )
 
 
 def get_env_bool(var_name):
@@ -31,17 +36,16 @@ SPLT_DIR = 'splits'
 OHLC_DIR = 'ohlc'
 SENT_DIR = 'sentiment'
 INTRA_DIR = 'intraday'
+IDX_DIR = 'indices'
 # providers
-IEX_DIR = 'iexcloud'
 POLY_DIR = 'polygon'
-TWIT_DIR = 'stocktwits'
+ALPACA_DIR = 'alpaca'
 # models
 MODELS_DIR = 'models'
 
 folders = {
-    'iexcloud': IEX_DIR,
     'polygon': POLY_DIR,
-    'stocktwits': TWIT_DIR
+    'alpaca': ALPACA_DIR
 }
 
 # Column Names
@@ -71,6 +75,7 @@ TRADES = 'Trades'
 
 # Time
 TZ = timezone('US/Eastern')
+UTC = timezone('UTC')
 DATE_FMT = '%Y-%m-%d'
 TIME_FMT = '%H:%M'
 PRECISE_TIME_FMT = '%H:%M:%S'
@@ -79,7 +84,6 @@ PRECISE_TIME_FMT = '%H:%M:%S'
 POS = 'Pos'
 NEG = 'Neg'
 DELTA = 'Delta'
-TWIT_RATE = 175
 
 # Unemployment
 UN_RATE = 'UnRate'
@@ -105,7 +109,7 @@ ABS_TOL = vbt.utils.math_.abs_tol
 
 # Model
 MAX_MODEL_AGE_DAYS = 90  # 3 months
-MIN_MODEL_ACCURACY = 0.95
+MIN_MODEL_ACCURACY = 0.925
 
 # Misc
 POLY_CRYPTO_SYMBOLS = [
@@ -113,6 +117,8 @@ POLY_CRYPTO_SYMBOLS = [
     'X%3ALTCUSD', 'X%3AXMRUSD', 'X%3AIOTUSD'
 ]
 
+ALPC_CRYPTO_SYMBOLS = ['BTC/USD', 'ETH/USD', 'LTC/USD']
+
 SENTIMENT_SYMBOLS_IGNORE = {
     'SPYD', 'VWDRY', 'BPMP',
     'FOX', 'YYY', 'SDIV',
@@ -128,6 +134,8 @@ FEW = 3
 FEW_DAYS = str(FEW) + 'd'
 SCRIPT_FAILURE_THRESHOLD = 0.95
 
+ALPACA_FREE_DELAY = 0.5
+
 # Exchanges
 BINANCE = 'BINANCE'
 KRAKEN = 'KRAKEN'
@@ -152,7 +160,7 @@ class PathFinder:
             'symbols.csv'
         )
 
-    def get_dividends_path(self, symbol, provider='iexcloud'):
+    def get_dividends_path(self, symbol, provider=POLY_DIR):
         # given a symbol
         # return the path to its csv
         return os.path.join(
@@ -162,7 +170,7 @@ class PathFinder:
             f'{symbol.upper()}.csv'
         )
 
-    def get_splits_path(self, symbol, provider='iexcloud'):
+    def get_splits_path(self, symbol, provider=POLY_DIR):
         # given a symbol
         # return the path to its stock splits
         return os.path.join(
@@ -172,7 +180,7 @@ class PathFinder:
             f'{symbol.upper()}.csv'
         )
 
-    def get_ohlc_path(self, symbol, provider='iexcloud'):
+    def get_ohlc_path(self, symbol, provider=POLY_DIR):
         # given a symbol
         # return the path to its ohlc data
         return os.path.join(
@@ -182,17 +190,7 @@ class PathFinder:
             f'{symbol.upper()}.csv'
         )
 
-    def get_sentiment_path(self, symbol, provider='stocktwits'):
-        # given a symbol
-        # return the path to its social sentiment data
-        return os.path.join(
-            DATA_DIR,
-            SENT_DIR,
-            folders[provider],
-            f'{symbol.upper()}.csv'
-        )
-
-    def get_intraday_path(self, symbol, date, provider='iexcloud'):
+    def get_intraday_path(self, symbol, date, provider=POLY_DIR):
         # given a symbol,
         # return the path to its intraday ohlc data
         return os.path.join(
@@ -241,6 +239,13 @@ class PathFinder:
             'orders.csv'
         )
 
+    def get_new_orders_path(self, provider):
+        return os.path.join(
+            DATA_DIR,
+            'orders',
+            f'{provider}.csv'
+        )
+
     def get_api_path(self, endpoint):
         return os.path.join(
             DATA_DIR,
@@ -248,6 +253,13 @@ class PathFinder:
             f'{endpoint}.json',
         )
 
+    def get_ndx_path(self):
+        return os.path.join(
+            DATA_DIR,
+            IDX_DIR,
+            'ndx.csv'
+        )
+
     def get_all_paths(self, path, truncate=False):
         # given a path, get all sub paths
         paths = []
