@@ -5,7 +5,9 @@ import base64
 import hashlib
 import requests
 import urllib.parse
+from time import sleep
 from binance import Client
+from typing import Iterable, Union, Optional
 from binance.helpers import round_step_size
 from dotenv import load_dotenv, find_dotenv
 import Constants as C
@@ -13,10 +15,100 @@ load_dotenv(find_dotenv('config.env'))
 
 
 class CEX:
-    def create_pair(self, base, quote):
+    def create_pair(self, base: str, quote: str) -> str:
         return f'{base}{quote}'
 
 
+class AlpacaEx(CEX):
+    def __init__(
+            self,
+            token: Optional[str] = os.environ.get('ALPACA'),
+            secret: Optional[str] = os.environ.get('ALPACA_SECRET'),
+            paper: bool = False
+    ) -> None:
+        super().__init__()
+        self.base = (f'https://{"paper-" if paper or C.TEST else ""}'
+                     'api.alpaca.markets')
+        self.version = 'v2'
+        self.token = os.environ.get(
+            'ALPACA_PAPER') if paper or C.TEST else token
+        self.secret = os.environ.get(
+            'ALPACA_PAPER_SECRET') if paper or C.TEST else secret
+        if not (self.token and self.secret):
+            raise Exception('missing Alpaca credentials')
+
+    def fill_orders(
+            self,
+            symbols: Iterable[str],
+            func: callable,
+            **kwargs: dict[str, any]
+    ) -> list[dict[str, any]]:
+        pending_orders = set()
+        completed_orders = []
+        for symbol in symbols:
+            order = func(symbol, **kwargs)
+            if order['status'] == 'filled':
+                completed_orders.append(order)
+            else:
+                pending_orders.add(order['id'])
+        while pending_orders:
+            for id in list(pending_orders):
+                order = self.get_order(id)
+                if order['status'] == 'filled':
+                    completed_orders.append(order)
+                    pending_orders.discard(id)
+            sleep(1)
+        return completed_orders
+
+    def make_request(
+            self,
+            method: str,
+            route: str,
+            payload: Optional[dict[str, any]] = {}
+    ) -> any:
+        parts = [self.base, self.version, route]
+        url = '/'.join(parts)
+        headers = {
+            "accept": "application/json",
+            "APCA-API-KEY-ID": self.token,
+            "APCA-API-SECRET-KEY": self.secret
+        }
+        response = requests.request(method, url, json=payload, headers=headers)
+        if response.ok:
+            return response.json()
+        else:
+            raise Exception(response.text)
+
+    def get_positions(self) -> any:
+        return self.make_request('GET', 'positions')
+
+    def close_position(self, symbol: str) -> any:
+        return self.make_request('DELETE', f'positions/{symbol}')
+
+    def get_order(self, id: str) -> any:
+        return self.make_request('GET', f'orders/{id}')
+
+    def get_account(self) -> any:
+        return self.make_request('GET', 'account')
+
+    def create_order(
+            self,
+            symbol: str,
+            side: str,
+            notional: Union[int, float, str]
+    ) -> any:
+        payload = {
+            'symbol': symbol,
+            'side': side.lower(),
+            'type': 'market',
+            'notional': str(notional),
+            'time_in_force': (
+                'gtc' if symbol in C.ALPC_CRYPTO_SYMBOLS else 'day'
+            )
+        }
+        return self.make_request('POST', 'orders', payload)
+
+
 class Kraken(CEX):
     def __init__(self, key=None, secret=None, test=False):
         super().__init__()
@@ -313,7 +405,10 @@ class Binance(CEX):
         params[quantity_label] = quantity
 
         params['side'] = side
-        fx = self.client.create_test_order if test else self.client.create_order
+        fx = (
+            self.client.create_test_order
+            if test else self.client.create_order
+        )
 
         order = fx(**params)
         return order
