@@ -15,39 +15,33 @@ class TestPathFinder():
 
     def test_get_dividends_path(self):
         assert finder.get_dividends_path(
-            'aapl') == 'data/dividends/iexcloud/AAPL.csv'
+            'aapl') == 'data/dividends/polygon/AAPL.csv'
         assert finder.get_dividends_path(
-            'AMD') == 'data/dividends/iexcloud/AMD.csv'
+            'AMD') == 'data/dividends/polygon/AMD.csv'
         assert finder.get_dividends_path(
             'TSLA', 'polygon') == 'data/dividends/polygon/TSLA.csv'
 
     def test_get_splits_path(self):
         assert finder.get_splits_path(
-            'aapl') == 'data/splits/iexcloud/AAPL.csv'
+            'aapl') == 'data/splits/polygon/AAPL.csv'
         assert finder.get_splits_path(
-            'AMD') == 'data/splits/iexcloud/AMD.csv'
+            'AMD') == 'data/splits/polygon/AMD.csv'
         assert finder.get_splits_path(
             'TSLA', 'polygon') == 'data/splits/polygon/TSLA.csv'
 
-    def test_get_sentiment_path(self):
-        assert finder.get_sentiment_path(
-            'aapl') == 'data/sentiment/stocktwits/AAPL.csv'
-        assert finder.get_sentiment_path(
-            'AMD') == 'data/sentiment/stocktwits/AMD.csv'
-
     def test_get_ohlc_path(self):
-        assert finder.get_ohlc_path('aapl') == 'data/ohlc/iexcloud/AAPL.csv'
-        assert finder.get_ohlc_path('AMD') == 'data/ohlc/iexcloud/AMD.csv'
+        assert finder.get_ohlc_path('aapl') == 'data/ohlc/polygon/AAPL.csv'
+        assert finder.get_ohlc_path('AMD') == 'data/ohlc/polygon/AMD.csv'
         assert finder.get_ohlc_path(
             'TSLA', 'polygon') == 'data/ohlc/polygon/TSLA.csv'
 
     def test_get_intraday_path(self):
         assert finder.get_intraday_path(
             'aapl', '2020-01-01'
-        ) == 'data/intraday/iexcloud/AAPL/2020-01-01.csv'
+        ) == 'data/intraday/polygon/AAPL/2020-01-01.csv'
         assert finder.get_intraday_path(
             'AMD', '2020-01-01'
-        ) == 'data/intraday/iexcloud/AMD/2020-01-01.csv'
+        ) == 'data/intraday/polygon/AMD/2020-01-01.csv'
         assert finder.get_intraday_path(
             'TSLA', '2020-01-01', 'polygon'
         ) == 'data/intraday/polygon/TSLA/2020-01-01.csv'
