@@ -1,35 +1,23 @@
 import os
-import sys
 import pytest
 from time import sleep, time
 from random import choice
 import pandas as pd
-sys.path.append('hyperdrive')
-from DataSource import MarketData, IEXCloud, Polygon, \
-                        StockTwits, LaborStats, Glassnode  # noqa autopep8
-import Constants as C  # noqa autopep8
-from Workflow import Flow  # noqa autopep8
-from Utils import SwissArmyKnife  # noqa autopep8
+from hyperdrive.DataSource import MarketData, Indices, Polygon, \
+                        LaborStats, Glassnode, AlpacaData  # noqa autopep8
+import hyperdrive.Constants as C  # noqa autopep8
+from hyperdrive.Workflow import Flow  # noqa autopep8
+from hyperdrive.Utils import SwissArmyKnife  # noqa autopep8
 
 
-md = MarketData()
-iex = IEXCloud(test=True)
-iex_intra = IEXCloud()
-poly = Polygon()
-twit = StockTwits()
-twit.token = ''
-bls = LaborStats()
-glass = Glassnode()
 flow = Flow()
 knife = SwissArmyKnife()
-
-iex = knife.use_dev(iex)
-iex_intra = knife.use_dev(iex_intra)
-md = knife.use_dev(md)
-poly = knife.use_dev(poly)
-twit = knife.use_dev(twit)
-bls = knife.use_dev(bls)
-glass = knife.use_dev(glass)
+md = knife.use_dev(MarketData())
+idc = knife.use_dev(Indices())
+alpc = knife.use_dev(AlpacaData(paper=True))
+poly = knife.use_dev(Polygon())
+bls = knife.use_dev(LaborStats())
+glass = knife.use_dev(Glassnode(use_cookies=True))
 
 exp_symbols = ['AMZN', 'META', 'NFLX']
 retries = 10
@@ -85,7 +73,7 @@ class TestMarketData:
             os.rename(div_path, temp_path)
 
         for _ in range(retries):
-            iex.save_dividends(
+            poly.save_dividends(
                 symbol=symbol, timeframe='5y', retries=1, delay=0)
             if not md.reader.check_file_exists(div_path):
                 delay = choice(range(5, 10))
@@ -124,7 +112,7 @@ class TestMarketData:
             assert col_in_df if curr_idx < sel_idx else not col_in_df
 
     def test_save_splits(self):
-        symbol = 'AAPL'
+        symbol = 'CMG'
         splt_path = md.finder.get_splits_path(symbol)
         temp_path = f'{splt_path}_TEMP'
 
@@ -132,7 +120,7 @@ class TestMarketData:
             os.rename(splt_path, temp_path)
 
         for _ in range(retries):
-            iex.save_splits(symbol=symbol, timeframe='5y', retries=1, delay=0)
+            poly.save_splits(symbol=symbol, timeframe='2y', retries=1, delay=0)
             if not md.reader.check_file_exists(splt_path):
                 delay = choice(range(5, 10))
                 sleep(delay)
@@ -142,74 +130,12 @@ class TestMarketData:
         assert md.reader.check_file_exists(splt_path)
         assert md.reader.store.modified_delta(splt_path).total_seconds() < 60
         df = md.reader.load_csv(splt_path)
-        assert {C.EX, C.DEC, C.RATIO}.issubset(df.columns)
+        assert {C.EX, C.RATIO}.issubset(df.columns)
         assert len(df) > 0
 
         if os.path.exists(temp_path):
             os.rename(temp_path, splt_path)
 
-    def test_get_social_sentiment(self):
-        df = md.get_social_sentiment('TSLA')
-        assert len(df) > 0
-        assert {C.TIME, C.POS, C.NEG}.issubset(df.columns)
-
-    def test_get_social_volume(self):
-        df = md.get_social_volume('TSLA')
-        assert len(df) > 0
-        assert {C.TIME, C.VOL, C.DELTA}.issubset(df.columns)
-
-    def test_save_social_sentiment(self):
-        symbol = 'ADBE'
-        sent_path = md.finder.get_sentiment_path(symbol)
-        temp_path = f'{sent_path}_TEMP'
-
-        if os.path.exists(sent_path):
-            os.rename(sent_path, temp_path)
-
-        twit.save_social_sentiment(
-            symbol=symbol, timeframe='1d', retries=1, delay=0)
-
-        assert md.reader.check_file_exists(sent_path)
-        assert md.reader.store.modified_delta(sent_path).total_seconds() < 60
-        df = md.reader.load_csv(sent_path)
-        assert {C.TIME, C.POS, C.NEG, C.VOL, C.DELTA}.issubset(df.columns)
-        assert len(df) > 0
-
-        if os.path.exists(temp_path):
-            os.rename(temp_path, sent_path)
-
-    def test_standardize_sentiment(self):
-        columns = ['timestamp', 'bullish', 'bearish']
-        new_cols = [C.TIME, C.POS, C.NEG]
-        sel_idx = 2
-        selected = columns[sel_idx:]
-        df = pd.DataFrame({column: [0] for column in columns})
-        standardized = md.standardize_sentiment('AAPL', df)
-        for column in new_cols:
-            assert column in standardized
-
-        df.drop(columns=selected, inplace=True)
-        standardized = md.standardize_sentiment('AAPL', df)
-        for curr_idx, column in enumerate(new_cols):
-            col_in_df = column in standardized
-            assert col_in_df if curr_idx < sel_idx else not col_in_df
-
-    def test_standardize_volume(self):
-        columns = ['timestamp', 'volume_score', 'volume_change']
-        new_cols = [C.TIME, C.VOL, C.DELTA]
-        sel_idx = 2
-        selected = columns[sel_idx:]
-        df = pd.DataFrame({column: [0] for column in columns})
-        standardized = md.standardize_volume('AAPL', df)
-        for column in new_cols:
-            assert column in standardized
-
-        df.drop(columns=selected, inplace=True)
-        standardized = md.standardize_volume('AAPL', df)
-        for curr_idx, column in enumerate(new_cols):
-            col_in_df = column in standardized
-            assert col_in_df if curr_idx < sel_idx else not col_in_df
-
     def test_standardize_ohlc(self):
         columns = ['date', 'open', 'high', 'low', 'close', 'volume']
         new_cols = [C.TIME, C.OPEN, C.HIGH, C.LOW, C.CLOSE, C.VOL]
@@ -235,7 +161,7 @@ class TestMarketData:
             os.rename(ohlc_path, temp_path)
 
         for _ in range(retries):
-            iex.save_ohlc(symbol=symbol, timeframe='1m', retries=1, delay=0)
+            poly.save_ohlc(symbol=symbol, timeframe='1m', retries=1, delay=0)
             if not md.reader.check_file_exists(ohlc_path):
                 delay = choice(range(5, 10))
                 sleep(delay)
@@ -253,12 +179,13 @@ class TestMarketData:
             os.rename(temp_path, ohlc_path)
 
     def test_save_intraday(self):
+        sleep(C.POLY_FREE_DELAY)
         symbol = 'NFLX'
         timeframe = '4d'
         dates = md.traveller.dates_in_range(timeframe)
         intra_paths = [md.finder.get_intraday_path(
             symbol, date) for date in dates]
-        filenames = set(iex_intra.save_intraday(
+        filenames = set(poly.save_intraday(
             symbol=symbol, timeframe=timeframe))
         intersection = filenames.intersection(intra_paths)
         assert intersection
@@ -271,7 +198,7 @@ class TestMarketData:
             os.remove(path)
 
     def test_get_ohlc(self):
-        df = md.get_ohlc('NFLX', '2m')
+        df = md.get_ohlc('NFLX', '5y')
         assert {C.TIME, C.OPEN, C.HIGH, C.LOW,
                 C.CLOSE, C.VOL}.issubset(df.columns)
         assert len(df) > 0
@@ -315,62 +242,62 @@ class TestMarketData:
     def test_save_sopr(self):
         assert 'sopr.csv' in md.save_sopr()
 
+    def test_get_ndx(self):
+        ndx = md.get_ndx()
+        assert {C.TIME, C.SYMBOL, C.DELTA}.issubset(ndx.columns)
+        assert 'AAPL' in set(ndx[C.SYMBOL])
+        assert (ndx[C.DELTA] == '+').all()
 
-class TestIEXCloud:
+    def test_standardize_ndx(self):
+        nonstd = pd.DataFrame({
+            C.TIME: ['2020-01-03', '2020-01-01', '2020-01-02'],
+            C.SYMBOL: ['AAPL', 'NFLX', 'AAPL'],
+            C.DELTA: ['-', '+', '+'],
+        })
+        std = pd.DataFrame({
+            C.TIME: ['2020-01-01'],
+            C.SYMBOL: ['NFLX'],
+            C.DELTA: ['+'],
+        })
+        assert md.standardize_ndx(nonstd).equals(std)
+
+    def test_save_ndx(self):
+        assert 'ndx.csv' in md.save_ndx()
+
+
+class TestIndices:
     def test_init(self):
-        assert type(iex).__name__ == 'IEXCloud'
-        assert hasattr(iex, 'base')
-        assert hasattr(iex, 'version')
-        assert hasattr(iex, 'token')
-        assert hasattr(iex, 'provider')
+        assert isinstance(idc, Indices)
 
-    def test_get_dividends(self):
-        df = []
+    def test_get_ndx(self):
+        ndx = idc.get_ndx()
+        assert {C.TIME, C.SYMBOL, C.DELTA}.issubset(ndx.columns)
+        assert 'AAPL' in set(ndx[C.SYMBOL])
+        assert (ndx[C.DELTA] == '+').all()
 
-        for i in range(retries):
-            if not len(df):
-                df = iex.get_dividends(symbol='AAPL', timeframe='5y')
-                if not i:
-                    delay = choice(range(5, 10))
-                    sleep(delay)
-            else:
-                break
 
-        assert len(df) > 0
-        assert {C.EX, C.PAY, C.DEC, C.DIV}.issubset(df.columns)
-
-    def test_get_splits(self):
-        df1, df2 = [], []
-        for i in range(retries):
-            if not (len(df1) or len(df2)):
-                df1 = iex.get_splits(symbol='AAPL', timeframe='5y')
-                df2 = iex.get_splits(symbol='NFLX', timeframe='5y')
-                if not i:
-                    delay = choice(range(5, 10))
-                    sleep(delay)
-            else:
-                break
-
-        assert len(df1) or len(df2)
-        assert {C.EX, C.DEC, C.RATIO}.issubset(
-            df1.columns) or {C.EX, C.DEC, C.RATIO}.issubset(df2.columns)
+class TestAlpaca:
+    def test_init(self):
+        assert isinstance(alpc, AlpacaData)
+        assert hasattr(alpc, 'base')
+        assert hasattr(alpc, 'token')
+        assert hasattr(alpc, 'secret')
+        assert hasattr(alpc, 'provider')
+        assert hasattr(alpc, 'free')
 
     def test_get_ohlc(self):
-        df = iex.get_ohlc(symbol='AAPL', timeframe='1m')
-        assert {C.TIME, C.OPEN, C.HIGH, C.LOW,
-                C.CLOSE, C.VOL}.issubset(df.columns)
-        assert len(df) > 10
-
-    def test_get_intraday(self):
-        df = pd.concat(iex_intra.get_intraday(symbol='AAPL', timeframe='1w'))
-        assert {C.TIME, C.OPEN, C.HIGH, C.LOW,
-                C.CLOSE, C.VOL}.issubset(df.columns)
-        assert len(df) > 1000
+        if not flow.is_any_workflow_running():
+            df = alpc.get_ohlc(symbol='AAPL', timeframe='1m')
+            assert {C.TIME, C.OPEN, C.HIGH, C.LOW,
+                    C.CLOSE, C.VOL, C.AVG}.issubset(df.columns)
+            assert len(df) > 10
+        else:
+            print('Skipping Alpaca OHLC test because update in progress')
 
 
 class TestPolygon:
     def test_init(self):
-        assert type(poly).__name__ == 'Polygon'
+        assert isinstance(poly, Polygon)
         assert hasattr(poly, 'client')
         assert hasattr(poly, 'provider')
 
@@ -423,35 +350,18 @@ class TestPolygon:
 
         then = time()
         poly.log_api_call_time()
-        poly.obey_free_limit()
+        poly.obey_free_limit(C.POLY_FREE_DELAY)
         now = time()
         assert now - then > C.POLY_FREE_DELAY
 
 
-class TestStockTwits:
-    def test_init(self):
-        assert type(twit).__name__ == 'StockTwits'
-        assert hasattr(twit, 'provider')
-        assert hasattr(twit, 'token')
-
-    def test_get_social_volume(self):
-        df = twit.get_social_volume(symbol='TSLA')
-        assert len(df) > 30
-        assert {C.TIME, C.VOL, C.DELTA}.issubset(df.columns)
-
-    def test_get_social_sentiment(self):
-        df = twit.get_social_sentiment(symbol='TSLA')
-        assert len(df) > 30
-        assert {C.TIME, C.POS, C.NEG}.issubset(df.columns)
-
-
 class TestLaborStats:
     def test_init(self):
         assert type(bls).__name__ == 'LaborStats'
-        assert hasattr(iex, 'base')
-        assert hasattr(iex, 'version')
-        assert hasattr(iex, 'token')
-        assert hasattr(iex, 'provider')
+        assert hasattr(bls, 'base')
+        assert hasattr(bls, 'version')
+        assert hasattr(bls, 'token')
+        assert hasattr(bls, 'provider')
 
     def test_get_unemployment_rate(self):
         df = bls.get_unemployment_rate(timeframe='2y')
