@@ -1,7 +1,7 @@
 import sys
 import pytest
 from datetime import datetime
-sys.path.append('src')
+sys.path.append('hyperdrive')
 from Workflow import Flow  # noqa autopep8
 
 flow = Flow()
@@ -16,4 +16,4 @@ class TestWorkFlow:
             assert flow.get_workflow_start_time('build')
 
     def test_is_workflow_running(self):
-        assert not flow.is_workflow_running('sentiment1')
+        assert not flow.is_workflow_running('unrate')
